@@ -26,7 +26,12 @@ function modal_success(modal, response) {
 function modal_error(modal, response) {
     flask_response = JSON.parse(response.responseText);
     modal.find('.modal-title').html('<a href="#" class="btn btn-rounded btn-danger">Error</a>');
-    modal.find('.modal-body').html(flask_response.message + `<span class="badge badge-pill badge-brand ml-1">` + response.statusText + `</span>`);
+    modal.find('.modal-body').html(flask_response.message + 
+        `<span class="badge badge-pill badge-brand ml-1">` + response.statusText + `</span>
+        <br>
+        <div class="alert alert-danger mt-1 mb-2" style="white-space: pre-wrap" role="alert">`+flask_response.traceback+
+        `</div>`
+        );
     modal.find('.modal-footer').hide();
 }
 
@@ -40,7 +45,18 @@ function refresh_dashboard() {
     }
 }
 
-function ajax_action(request_type, action_url, _data, table, modal) {
+function post_worker_suspend_resume(){
+    current = $('#suspendresume').data('action')
+    if(current == 'suspendall'){
+        $('#suspendresume').data('action', 'resumeall');
+        $('#suspendresume').text('Resume All Workers');
+    }else{
+        $('#suspendresume').data('action', 'suspendall');
+        $('#suspendresume').text('Suspend All Workers');
+    }
+}
+
+function ajax_action(request_type, action_url, _data, table, modal, post_success = undefined) {
     $.ajax({
         type: request_type,
         url: action_url,
@@ -53,9 +69,12 @@ function ajax_action(request_type, action_url, _data, table, modal) {
             setTimeout(function () {
                 $('#confirmation').modal('hide');
             }, 2000);
+            if (post_success !== undefined){
+                post_success();
+            }
         },
-        error: function (rs, e) {
-            modal_error(modal, rs)
+        error: function (jqXHR, textStatus, errorThrown) {
+            modal_error(modal, jqXHR);
         }
     });
 }
@@ -82,9 +101,9 @@ function action_modal_onshow() {
             } else if (action === 'delete') {
                 update_modal_info(modal, 'Confirm to delete ' + target_id, target_class, target_id, action, 'Queue will be deleted along with all jobs on it!')
             } else if (action === 'deleteall') {
-                update_modal_info(modal, 'Confirm to delete these queues ' + get_checked_queues(), target_class, target_id, action, 'These queues will be deleted along with all jobs on it!')
+                update_modal_info(modal, 'Confirm to delete these queues ' + get_checked_queues().join(", "), target_class, target_id, action, 'These queues will be deleted along with all jobs on it!')
             } else if (action === 'emptyall') {
-                update_modal_info(modal, 'Confirm to empty these queues ' + get_checked_queues(), target_class, target_id, action, 'All the queued jobs will be removed from these queues!')
+                update_modal_info(modal, 'Confirm to empty these queues ' + get_checked_queues().join(", "), target_class, target_id, action, 'All the queued jobs will be removed from these queues!')
             }
         } else if (target_class === 'job') {
             var queue_for_job = button.data('jobqueue')
@@ -95,17 +114,21 @@ function action_modal_onshow() {
             } else if (action === "cancel") {
                 update_modal_info(modal, 'Confirm to cancel ' + target_id, target_class, target_id, action, 'Job will be cancelled and never be executed or inspected!')
             } else if (action === "cancelall") {
-                update_modal_info(modal, 'Confirm to cancel all jobs on ' + get_checked_queues(), target_class, target_id, action, 'Jobs will be cancelled and never be executed or inspected!')
+                update_modal_info(modal, 'Confirm to cancel all jobs on ' + get_checked_queues().join(", "), target_class, target_id, action, 'Jobs will be cancelled and never be executed or inspected!')
             } else if (action === "deleteall") {
-                update_modal_info(modal, 'Confirm to delete all jobs on ' + get_checked_queues(), target_class, target_id, action, 'Jobs will be permanently deleted from Redis!')
+                update_modal_info(modal, 'Confirm to delete all jobs on ' + get_checked_queues().join(", "), target_class, target_id, action, 'Jobs will be permanently deleted from Redis!')
             } else if (action === "requeueall") {
-                update_modal_info(modal, 'Confirm to requeue all failed jobs from ' + get_checked_queues(), target_class, target_id, action, 'All failed jobs will be removed from their queue failed job registry and put on their respective original queues again')
+                update_modal_info(modal, 'Confirm to requeue all failed jobs from ' + get_checked_queues().join(", "), target_class, target_id, action, 'All failed jobs will be removed from their queue failed job registry and put on their respective original queues again')
             }
         } else if (target_class === 'worker') {
             if (action === 'delete') {
-                update_modal_info(modal, 'Confirm to delete ' + target_id, target_class, target_id, action, 'Worker will be sent SIGINT to request warm shutdown, any currently executing tasks will be completed first.')
+                update_modal_info(modal, 'Confirm to delete ' + target_id, target_class, target_id, action, 'Worker on same instance will be sent SIGINT to request warm shutdown, any currently executing tasks will be completed first.')
             } else if (action === 'deleteall') {
-                update_modal_info(modal, 'Confirm to delete all workers', target_class, target_id, action, 'All workers will be sent SIGINT to request warm shutdown, any currently executing tasks will be completed first.')
+                update_modal_info(modal, 'Confirm to delete all workers', target_class, target_id, action, 'All workers on same instance will be sent SIGINT to request warm shutdown, any currently executing tasks will be completed first.')
+            } else if(action == 'suspendall'){
+                update_modal_info(modal, 'Confirm to suspend all workers', target_class, target_id, action, 'All workers will be suspended, no jobs will be executed now, any currently executing tasks will be completed first. Resume workers to make them work again.')                
+            } else if(action == 'resumeall'){
+                update_modal_info(modal, 'Confirm to resume all workers', target_class, target_id, action, 'All workers will be resumed and will start to work again.')                
             }
         }
     })
@@ -138,7 +161,7 @@ function get_currently_selected_redis_index() {
 }
 
 
-function setup_queues_datatable(site_map) {
+function setup_queues_datatable(nunjucks_urls, site_map) {
 
     queues_table = $('#queues_table').DataTable({
         "processing": "True",
@@ -156,6 +179,22 @@ function setup_queues_datatable(site_map) {
             },
             "dataSrc": "data",
             "dataType": "json",
+            "error": function (jqXHR, textStatus, errorThrown) {
+                $.get({
+                    url: nunjucks_urls['error'],
+                    cache: false
+                }).then(function(error_template){
+                    rendered_template = nunjucks.renderString(
+                            error_template, 
+                            { 
+                                'error_info': JSON.parse(jqXHR.responseText),
+                                'textStatus': textStatus,
+                                'errorThrown': errorThrown,
+                            }
+                        );
+                    $('#main_content').html(rendered_template);
+                })
+            }
         },
         "columns": [
             {
@@ -249,7 +288,7 @@ function on_redis_memory_refresh(site_map){
     });
 }
 
-function setup_worker_datatable(site_map) {
+function setup_worker_datatable(nunjucks_urls, site_map) {
     workers_table = $('#workers_table').DataTable({
         "processing": "True",
         "language": {
@@ -266,6 +305,22 @@ function setup_worker_datatable(site_map) {
             },
             "dataSrc": "data",
             "dataType": "json",
+            "error": function (jqXHR, textStatus, errorThrown) {
+                $.get({
+                    url: nunjucks_urls['error'],
+                    cache: false
+                }).then(function(error_template){
+                    rendered_template = nunjucks.renderString(
+                            error_template, 
+                            { 
+                                'error_info': JSON.parse(jqXHR.responseText),
+                                'textStatus': textStatus,
+                                'errorThrown': errorThrown,
+                            }
+                        );
+                    $('#main_content').html(rendered_template);
+                })
+            }
         },
         "columns": [
             {
@@ -273,7 +328,7 @@ function setup_worker_datatable(site_map) {
                 render: function (data, type, row, meta) {
                     if (type === 'display') {
                         data = `
-                <a href="#" data-toggle="modal" data-target="#worker_info"
+                <a href="#" data-toggle="modal" data-target="#infomodal"
                 data-worker="`+ data + `">` + data + `</a>
                 `;
                     }
@@ -316,14 +371,14 @@ function setup_worker_datatable(site_map) {
     });
 
     function show_worker_modal(site_map){
-        $('#worker_info').on('show.bs.modal', function (event) {
+        $('#infomodal').on('show.bs.modal', function (event) {
             var button = $(event.relatedTarget); // Button that triggered the modal
             var worker_id = button.data('worker'); // Extract info from data-* attributes
             var modal = $(this);
     
             $.ajax({
                 url: site_map['rqmonitor.worker_info_api'],
-                data: { 'worker_id': worker_id, 'csrfmiddlewaretoken': '{{ csrf_token }}' },
+                data: { 'worker_id': worker_id},
                 success: function (data) {
                     modal.find('.modal-title').text('Showing Info for ' + worker_id)
                     modal.attr('name', worker_id)
@@ -346,6 +401,8 @@ function setup_worker_datatable(site_map) {
         });
     }
 
+    show_worker_modal(site_map);
+
     /*
     $('#workers_table tbody').on('click', '.worker_info_modal', function () {
       var row_clicked = workers_table.row($(this).parents('tr')).data();
@@ -354,12 +411,11 @@ function setup_worker_datatable(site_map) {
     */
 }
 
-function setup_jobs_datatable(job_info_url, site_map) {
+function setup_jobs_datatable(nunjucks_urls, site_map) {
     var job_result_ttl = "specifies how long (in seconds) successful jobs and their results are kept. Expired jobs will be automatically deleted. Defaults to 500 seconds.";
     var job_timeout = "specifies the maximum runtime of the job before it’s interrupted and marked as failed. Its default unit is seconds and it can be an integer or a string representing an integer(e.g. 2, '2'). Furthermore, it can be a string with specify unit including hour, minute, second (e.g. '1h', '3m', '5s').";
     var job_failure_ttl = " specifies how long (in seconds) failed jobs are kept (defaults to 1 year)";
     var job_ttl = "specifies the maximum queued time (in seconds) of the job before it’s discarded. This argument defaults to None (infinite TTL).";
-    var job_info_template;
 
     /*
     from flask and jinja2's perspective all templates to be rendered by nunjucks 
@@ -368,11 +424,9 @@ function setup_jobs_datatable(job_info_url, site_map) {
     */
 
     $.get({
-        url: job_info_url,
+        url: nunjucks_urls['job_info'],
         cache: false
-    }).then(function (data) {
-        job_info_template = data;
-
+    }).then(function (job_info_template) {
         jobs_table = $('#jobs_table').DataTable({
             "serverSide": "True",
             "pageLength": 50,
@@ -396,13 +450,28 @@ function setup_jobs_datatable(job_info_url, site_map) {
                     )
                 },
                 "dataSrc": "data",
-                dataType: "json",
+                "dataType": "json",
+                "error": function (jqXHR, textStatus, errorThrown) {
+                    $.get({
+                        url: nunjucks_urls['error'],
+                        cache: false
+                    }).then(function(error_template){
+                        rendered_template = nunjucks.renderString(
+                                error_template, 
+                                { 
+                                    'error_info': JSON.parse(jqXHR.responseText),
+                                    'textStatus': textStatus,
+                                    'errorThrown': errorThrown,
+                                }
+                            );
+                        $('#main_content').html(rendered_template);
+                    })
+                }
             },
             "columns": [
                 {
                     data: "job_info",
                     render: function (data, type, row, meta) {
-                        //alert(data['job_id']);
                         if (type === 'display') {
                             //data = nunjucks.render("{{ url_for('static', filename='nunjucks/job_info.html') }}", data)
                             data = nunjucks.renderString(job_info_template, { 'job_data': data });
@@ -457,9 +526,11 @@ function action_modal_onconfirm(site_map) {
         var task = $(this).closest('.modal').attr('task');
         var modal = $(this).closest('.modal');
 
+        modal.find('.modal-body').html('<span class="dashboard-spinner spinner-sm"></span>')
+
         if (target_class === 'queue') {
             if (task === 'empty') {
-                ajax_action("POST", site_map['rqmonitor.empty_queue_api'], { 'queue_id': target_id}, queues_table, modal);
+                ajax_action("POST", site_map['rqmonitor.empty_queue_api'], { 'queue_id': target_id}, queues_table, modal);           
             } else if (task === 'delete') {
                 ajax_action("POST", site_map['rqmonitor.delete_queue_api'], { 'queue_id': target_id}, queues_table, modal);
                 reload_sidebar_queues(site_map);
@@ -503,6 +574,12 @@ function action_modal_onconfirm(site_map) {
                 ajax_action("POST", site_map['rqmonitor.delete_workers_api'], {
                     'delete_all': "true",
                 }, workers_table, modal);
+            } else if (task === "suspendall") {
+                ajax_action("POST", site_map['rqmonitor.suspend_workers_api'], {
+                }, workers_table, modal, post_worker_suspend_resume);
+            } else if (task === "resumeall") {
+                ajax_action("POST", site_map['rqmonitor.resume_workers_api'], {
+                }, workers_table, modal, post_worker_suspend_resume);
             }
         }
 
@@ -525,7 +602,7 @@ function on_queue_selection_change(){
     });
 }  
 
-function on_click_jobs_dashboard(nunjucks_job_info_url, sitemap){
+function on_click_jobs_dashboard(nunjucks_urls, sitemap){
     $('#jobsmenu').on('show.bs.collapse', function (e) {
         if ($(this).is(e.target)) {
             $.ajax({
@@ -534,7 +611,7 @@ function on_click_jobs_dashboard(nunjucks_job_info_url, sitemap){
                 data: inject_globals(),
                 success: function (response) {
                     $('#main_dashboard').html(response);
-                    setup_jobs_datatable(nunjucks_job_info_url, site_map);
+                    setup_jobs_datatable(nunjucks_urls, site_map);
                 },
                 error: function (rs, e) {
                     $('#main_dashboard').html(`<strong>` + JSON.stringify(rs) + `</strong>`);
@@ -544,7 +621,7 @@ function on_click_jobs_dashboard(nunjucks_job_info_url, sitemap){
     });
 }
 
-function on_click_workers_dashboard(site_map){
+function on_click_workers_dashboard(nunjucks_urls, site_map){
     $('#workers_link').on('click', function () {
         $.ajax({
             type: "GET",
@@ -552,7 +629,7 @@ function on_click_workers_dashboard(site_map){
             data: inject_globals(),
             success: function (response) {
                 $('#main_dashboard').html(response);
-                setup_worker_datatable(site_map);
+                setup_worker_datatable(nunjucks_urls, site_map);
             },
             error: function (rs, e) {
                 $('#main_dashboard').html(`<strong>` + JSON.stringify(rs) + `</strong>`);
@@ -561,7 +638,7 @@ function on_click_workers_dashboard(site_map){
     });
 }
 
-function on_click_queues_dashboard(site_map){
+function on_click_queues_dashboard(nunjucks_urls, site_map){
     $('#queues_link').on('click', function () {
         $.ajax({
             type: "GET",
@@ -569,7 +646,7 @@ function on_click_queues_dashboard(site_map){
             data: inject_globals(),
             success: function (response) {
                 $('#main_dashboard').html(response);
-                setup_queues_datatable(site_map);
+                setup_queues_datatable(nunjucks_urls, site_map);
             },
             error: function (rs, e) {
                 $('#main_dashboard').html(`<strong>` + JSON.stringify(rs) + `</strong>`);
